*** WeiDU-185/zlib/inftrees.c	Fri Sep 27 03:25:32 2002
--- WeiDU/zlib/inftrees.c	Sat Sep  1 02:40:48 2007
***************
*** 1,454 ****
  /* inftrees.c -- generate Huffman trees for efficient decoding
!  * Copyright (C) 1995-2002 Mark Adler
   * For conditions of distribution and use, see copyright notice in zlib.h 
   */
  
  #include "zutil.h"
  #include "inftrees.h"
  
! #if !defined(BUILDFIXED) && !defined(STDC)
! #  define BUILDFIXED   /* non ANSI compilers may not accept inffixed.h */
! #endif
  
  const char inflate_copyright[] =
!    " inflate 1.1.4 Copyright 1995-2002 Mark Adler ";
  /*
    If you use the zlib library in a product, an acknowledgment is welcome
    in the documentation of your product. If for some reason you cannot
    include such an acknowledgment, I would appreciate that you keep this
    copyright string in the executable of your product.
   */
- struct internal_state  {int dummy;}; /* for buggy compilers */
  
! /* simplify the use of the inflate_huft type with some defines */
! #define exop word.what.Exop
! #define bits word.what.Bits
! 
! 
! local int huft_build OF((
!     uIntf *,            /* code lengths in bits */
!     uInt,               /* number of codes */
!     uInt,               /* number of "simple" codes */
!     const uIntf *,      /* list of base values for non-simple codes */
!     const uIntf *,      /* list of extra bits for non-simple codes */
!     inflate_huft * FAR*,/* result: starting table */
!     uIntf *,            /* maximum lookup bits (returns actual) */
!     inflate_huft *,     /* space for trees */
!     uInt *,             /* hufts used in space */
!     uIntf * ));         /* space for values */
! 
! /* Tables for deflate from PKZIP's appnote.txt. */
! local const uInt cplens[31] = { /* Copy lengths for literal codes 257..285 */
          3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
          35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0};
!         /* see note #13 above about 258 */
! local const uInt cplext[31] = { /* Extra bits for literal codes 257..285 */
!         0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 2, 2, 2, 2,
!         3, 3, 3, 3, 4, 4, 4, 4, 5, 5, 5, 5, 0, 112, 112}; /* 112==invalid */
! local const uInt cpdist[30] = { /* Copy offsets for distance codes 0..29 */
          1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
          257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
!         8193, 12289, 16385, 24577};
! local const uInt cpdext[30] = { /* Extra bits for distance codes */
!         0, 0, 0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
!         7, 7, 8, 8, 9, 9, 10, 10, 11, 11,
!         12, 12, 13, 13};
  
  /*
!    Huffman code decoding is performed using a multi-level table lookup.
!    The fastest way to decode is to simply build a lookup table whose
!    size is determined by the longest code.  However, the time it takes
!    to build this table can also be a factor if the data being decoded
!    is not very long.  The most common codes are necessarily the
!    shortest codes, so those codes dominate the decoding time, and hence
!    the speed.  The idea is you can have a shorter table that decodes the
!    shorter, more probable codes, and then point to subsidiary tables for
!    the longer codes.  The time it costs to decode the longer codes is
!    then traded against the time it takes to make longer tables.
! 
!    This results of this trade are in the variables lbits and dbits
!    below.  lbits is the number of bits the first level table for literal/
!    length codes can decode in one step, and dbits is the same thing for
!    the distance codes.  Subsequent tables are also less than or equal to
!    those sizes.  These values may be adjusted either when all of the
!    codes are shorter than that, in which case the longest code length in
!    bits is used, or when the shortest code is *longer* than the requested
!    table size, in which case the length of the shortest code in bits is
!    used.
! 
!    There are two different values for the two tables, since they code a
!    different number of possibilities each.  The literal/length table
!    codes 286 possible values, or in a flat code, a little over eight
!    bits.  The distance table codes 30 possible values, or a little less
!    than five bits, flat.  The optimum values for speed end up being
!    about one bit more than those, so lbits is 8+1 and dbits is 5+1.
!    The optimum values may differ though from machine to machine, and
!    possibly even between compilers.  Your mileage may vary.
   */
  
  
! /* If BMAX needs to be larger than 16, then h and x[] should be uLong. */
! #define BMAX 15         /* maximum bit length of any code */
! 
! local int huft_build(b, n, s, d, e, t, m, hp, hn, v)
! uIntf *b;               /* code lengths in bits (all assumed <= BMAX) */
! uInt n;                 /* number of codes (assumed <= 288) */
! uInt s;                 /* number of simple-valued codes (0..s-1) */
! const uIntf *d;         /* list of base values for non-simple codes */
! const uIntf *e;         /* list of extra bits for non-simple codes */
! inflate_huft * FAR *t;  /* result: starting table */
! uIntf *m;               /* maximum lookup bits, returns actual */
! inflate_huft *hp;       /* space for trees */
! uInt *hn;               /* hufts used in space */
! uIntf *v;               /* working area: values in order of bit length */
! /* Given a list of code lengths and a maximum table size, make a set of
!    tables to decode that set of codes.  Return Z_OK on success, Z_BUF_ERROR
!    if the given code set is incomplete (the tables are still built in this
!    case), or Z_DATA_ERROR if the input is invalid. */
! {
! 
!   uInt a;                       /* counter for codes of length k */
!   uInt c[BMAX+1];               /* bit length count table */
!   uInt f;                       /* i repeats in table every f entries */
!   int g;                        /* maximum code length */
!   int h;                        /* table level */
!   register uInt i;              /* counter, current code */
!   register uInt j;              /* counter */
!   register int k;               /* number of bits in current code */
!   int l;                        /* bits per table (returned in m) */
!   uInt mask;                    /* (1 << w) - 1, to avoid cc -O bug on HP */
!   register uIntf *p;            /* pointer into c[], b[], or v[] */
!   inflate_huft *q;              /* points to current table */
!   struct inflate_huft_s r;      /* table entry for structure assignment */
!   inflate_huft *u[BMAX];        /* table stack */
!   register int w;               /* bits before this table == (l * h) */
!   uInt x[BMAX+1];               /* bit offsets, then code stack */
!   uIntf *xp;                    /* pointer into x */
!   int y;                        /* number of dummy codes added */
!   uInt z;                       /* number of entries in current table */
! 
! 
!   /* Generate counts for each bit length */
!   p = c;
! #define C0 *p++ = 0;
! #define C2 C0 C0 C0 C0
! #define C4 C2 C2 C2 C2
!   C4                            /* clear c[]--assume BMAX+1 is 16 */
!   p = b;  i = n;
!   do {
!     c[*p++]++;                  /* assume all entries <= BMAX */
!   } while (--i);
!   if (c[0] == n)                /* null input--all zero length codes */
!   {
!     *t = (inflate_huft *)Z_NULL;
!     *m = 0;
!     return Z_OK;
!   }
! 
  
!   /* Find minimum and maximum length, bound *m by those */
!   l = *m;
!   for (j = 1; j <= BMAX; j++)
!     if (c[j])
        break;
!   k = j;                        /* minimum code length */
!   if ((uInt)l < j)
!     l = j;
!   for (i = BMAX; i; i--)
!     if (c[i])
        break;
!   g = i;                        /* maximum code length */
!   if ((uInt)l > i)
!     l = i;
!   *m = l;
! 
! 
!   /* Adjust last length count to fill out codes, if needed */
!   for (y = 1 << j; j < i; j++, y <<= 1)
!     if ((y -= c[j]) < 0)
!       return Z_DATA_ERROR;
!   if ((y -= c[i]) < 0)
!     return Z_DATA_ERROR;
!   c[i] += y;
! 
! 
!   /* Generate starting offsets into the value table for each length */
!   x[1] = j = 0;
!   p = c + 1;  xp = x + 2;
!   while (--i) {                 /* note that i == g from above */
!     *xp++ = (j += *p++);
!   }
! 
! 
!   /* Make a table of values in order of bit lengths */
!   p = b;  i = 0;
    do {
!     if ((j = *p++) != 0)
!       v[x[j]++] = i;
!   } while (++i < n);
!   n = x[g];                     /* set n to length of v */
! 
! 
!   /* Generate the Huffman codes and for each, make the table entries */
!   x[0] = i = 0;                 /* first Huffman code is zero */
!   p = v;                        /* grab values in bit order */
!   h = -1;                       /* no tables yet--level -1 */
!   w = -l;                       /* bits decoded == (l * h) */
!   u[0] = (inflate_huft *)Z_NULL;        /* just to keep compilers happy */
!   q = (inflate_huft *)Z_NULL;   /* ditto */
!   z = 0;                        /* ditto */
! 
!   /* go through the bit lengths (k already is bits in shortest code) */
!   for (; k <= g; k++)
!   {
!     a = c[k];
!     while (a--)
!     {
!       /* here i is the Huffman code of length k bits for value *p */
!       /* make tables up to required level */
!       while (k > w + l)
!       {
!         h++;
!         w += l;                 /* previous table always l bits */
! 
!         /* compute minimum size table less than or equal to l bits */
!         z = g - w;
!         z = z > (uInt)l ? l : z;        /* table size upper limit */
!         if ((f = 1 << (j = k - w)) > a + 1)     /* try a k-w bit table */
!         {                       /* too few codes for k-w bit table */
!           f -= a + 1;           /* deduct codes from patterns left */
!           xp = c + k;
!           if (j < z)
!             while (++j < z)     /* try smaller tables up to z bits */
!             {
!               if ((f <<= 1) <= *++xp)
!                 break;          /* enough codes to use up j bits */
!               f -= *xp;         /* else deduct codes from patterns */
!             }
!         }
!         z = 1 << j;             /* table entries for j-bit table */
! 
!         /* allocate new table */
!         if (*hn + z > MANY)     /* (note: doesn't matter for fixed) */
!           return Z_DATA_ERROR;  /* overflow of MANY */
!         u[h] = q = hp + *hn;
!         *hn += z;
! 
!         /* connect to last table, if there is one */
!         if (h)
!         {
!           x[h] = i;             /* save pattern for backing up */
!           r.bits = (Byte)l;     /* bits to dump before this table */
!           r.exop = (Byte)j;     /* bits in this table */
!           j = i >> (w - l);
!           r.base = (uInt)(q - u[h-1] - j);   /* offset to this table */
!           u[h-1][j] = r;        /* connect to last table */
          }
          else
!           *t = q;               /* first table is returned result */
!       }
  
!       /* set up table entry in r */
!       r.bits = (Byte)(k - w);
!       if (p >= v + n)
!         r.exop = 128 + 64;      /* out of values--invalid code */
!       else if (*p < s)
!       {
!         r.exop = (Byte)(*p < 256 ? 0 : 32 + 64);     /* 256 is end-of-block */
!         r.base = *p++;          /* simple code is just the value */
        }
-       else
-       {
-         r.exop = (Byte)(e[*p - s] + 16 + 64);/* non-simple--look up in lists */
-         r.base = d[*p++ - s];
        }
  
!       /* fill code-like entries with r */
!       f = 1 << (k - w);
!       for (j = i >> w; j < z; j += f)
!         q[j] = r;
! 
!       /* backwards increment the k-bit code i */
!       for (j = 1 << (k - 1); i & j; j >>= 1)
!         i ^= j;
!       i ^= j;
! 
!       /* backup over finished tables */
!       mask = (1 << w) - 1;      /* needed on HP, cc -O bug */
!       while ((i & mask) != x[h])
!       {
!         h--;                    /* don't need to update q */
!         w -= l;
!         mask = (1 << w) - 1;
!       }
!     }
!   }
! 
! 
!   /* Return Z_BUF_ERROR if we were given an incomplete table */
!   return y != 0 && g != 1 ? Z_BUF_ERROR : Z_OK;
! }
! 
! 
! int inflate_trees_bits(c, bb, tb, hp, z)
! uIntf *c;               /* 19 code lengths */
! uIntf *bb;              /* bits tree desired/actual depth */
! inflate_huft * FAR *tb; /* bits tree result */
! inflate_huft *hp;       /* space for trees */
! z_streamp z;            /* for messages */
! {
!   int r;
!   uInt hn = 0;          /* hufts used in space */
!   uIntf *v;             /* work area for huft_build */
! 
!   if ((v = (uIntf*)ZALLOC(z, 19, sizeof(uInt))) == Z_NULL)
!     return Z_MEM_ERROR;
!   r = huft_build(c, 19, 19, (uIntf*)Z_NULL, (uIntf*)Z_NULL,
!                  tb, bb, hp, &hn, v);
!   if (r == Z_DATA_ERROR)
!     z->msg = (char*)"oversubscribed dynamic bit lengths tree";
!   else if (r == Z_BUF_ERROR || *bb == 0)
!   {
!     z->msg = (char*)"incomplete dynamic bit lengths tree";
!     r = Z_DATA_ERROR;
!   }
!   ZFREE(z, v);
!   return r;
! }
! 
! 
! int inflate_trees_dynamic(nl, nd, c, bl, bd, tl, td, hp, z)
! uInt nl;                /* number of literal/length codes */
! uInt nd;                /* number of distance codes */
! uIntf *c;               /* that many (total) code lengths */
! uIntf *bl;              /* literal desired/actual bit depth */
! uIntf *bd;              /* distance desired/actual bit depth */
! inflate_huft * FAR *tl; /* literal/length tree result */
! inflate_huft * FAR *td; /* distance tree result */
! inflate_huft *hp;       /* space for trees */
! z_streamp z;            /* for messages */
! {
!   int r;
!   uInt hn = 0;          /* hufts used in space */
!   uIntf *v;             /* work area for huft_build */
! 
!   /* allocate work area */
!   if ((v = (uIntf*)ZALLOC(z, 288, sizeof(uInt))) == Z_NULL)
!     return Z_MEM_ERROR;
! 
!   /* build literal/length tree */
!   r = huft_build(c, nl, 257, cplens, cplext, tl, bl, hp, &hn, v);
!   if (r != Z_OK || *bl == 0)
!   {
!     if (r == Z_DATA_ERROR)
!       z->msg = (char*)"oversubscribed literal/length tree";
!     else if (r != Z_MEM_ERROR)
!     {
!       z->msg = (char*)"incomplete literal/length tree";
!       r = Z_DATA_ERROR;
!     }
!     ZFREE(z, v);
!     return r;
!   }
! 
!   /* build distance tree */
!   r = huft_build(c + nl, nd, 0, cpdist, cpdext, td, bd, hp, &hn, v);
!   if (r != Z_OK || (*bd == 0 && nl > 257))
!   {
!     if (r == Z_DATA_ERROR)
!       z->msg = (char*)"oversubscribed distance tree";
!     else if (r == Z_BUF_ERROR) {
! #ifdef PKZIP_BUG_WORKAROUND
!       r = Z_OK;
!     }
! #else
!       z->msg = (char*)"incomplete distance tree";
!       r = Z_DATA_ERROR;
!     }
!     else if (r != Z_MEM_ERROR)
!     {
!       z->msg = (char*)"empty distance tree with lengths";
!       r = Z_DATA_ERROR;
!     }
!     ZFREE(z, v);
!     return r;
! #endif
!   }
! 
!   /* done */
!   ZFREE(z, v);
!   return Z_OK;
  }
! 
! 
! /* build fixed tables only once--keep them here */
! #ifdef BUILDFIXED
! local int fixed_built = 0;
! #define FIXEDH 544      /* number of hufts used by fixed tables */
! local inflate_huft fixed_mem[FIXEDH];
! local uInt fixed_bl;
! local uInt fixed_bd;
! local inflate_huft *fixed_tl;
! local inflate_huft *fixed_td;
! #else
! #include "inffixed.h"
! #endif
! 
! 
! int inflate_trees_fixed(bl, bd, tl, td, z)
! uIntf *bl;               /* literal desired/actual bit depth */
! uIntf *bd;               /* distance desired/actual bit depth */
! inflate_huft * FAR *tl;  /* literal/length tree result */
! inflate_huft * FAR *td;  /* distance tree result */
! z_streamp z;             /* for memory allocation */
! {
! #ifdef BUILDFIXED
!   /* build fixed tables if not already */
!   if (!fixed_built)
!   {
!     int k;              /* temporary variable */
!     uInt f = 0;         /* number of hufts used in fixed_mem */
!     uIntf *c;           /* length list for huft_build */
!     uIntf *v;           /* work area for huft_build */
! 
!     /* allocate memory */
!     if ((c = (uIntf*)ZALLOC(z, 288, sizeof(uInt))) == Z_NULL)
!       return Z_MEM_ERROR;
!     if ((v = (uIntf*)ZALLOC(z, 288, sizeof(uInt))) == Z_NULL)
!     {
!       ZFREE(z, c);
!       return Z_MEM_ERROR;
      }
  
!     /* literal table */
!     for (k = 0; k < 144; k++)
!       c[k] = 8;
!     for (; k < 256; k++)
!       c[k] = 9;
!     for (; k < 280; k++)
!       c[k] = 7;
!     for (; k < 288; k++)
!       c[k] = 8;
!     fixed_bl = 9;
!     huft_build(c, 288, 257, cplens, cplext, &fixed_tl, &fixed_bl,
!                fixed_mem, &f, v);
! 
!     /* distance table */
!     for (k = 0; k < 30; k++)
!       c[k] = 5;
!     fixed_bd = 5;
!     huft_build(c, 30, 0, cpdist, cpdext, &fixed_td, &fixed_bd,
!                fixed_mem, &f, v);
! 
!     /* done */
!     ZFREE(z, v);
!     ZFREE(z, c);
!     fixed_built = 1;
!   }
! #endif
!   *bl = fixed_bl;
!   *bd = fixed_bd;
!   *tl = fixed_tl;
!   *td = fixed_td;
!   return Z_OK;
  }
--- 1,329 ----
  /* inftrees.c -- generate Huffman trees for efficient decoding
!  * Copyright (C) 1995-2006 Mark Adler
   * For conditions of distribution and use, see copyright notice in zlib.h
   */
  
  #include "zutil.h"
  #include "inftrees.h"
  
! #define MAXBITS 15
  
  const char inflate_copyright[] =
!    " inflate 1.2.3.3 Copyright 1995-2006 Mark Adler ";
  /*
    If you use the zlib library in a product, an acknowledgment is welcome
    in the documentation of your product. If for some reason you cannot
    include such an acknowledgment, I would appreciate that you keep this
    copyright string in the executable of your product.
   */
  
! /*
!    Build a set of tables to decode the provided canonical Huffman code.
!    The code lengths are lens[0..codes-1].  The result starts at *table,
!    whose indices are 0..2^bits-1.  work is a writable array of at least
!    lens shorts, which is used as a work area.  type is the type of code
!    to be generated, CODES, LENS, or DISTS.  On return, zero is success,
!    -1 is an invalid code, and +1 means that ENOUGH isn't enough.  table
!    on return points to the next available entry's address.  bits is the
!    requested root table index bits, and on return it is the actual root
!    table index bits.  It will differ if the request is greater than the
!    longest code or if it is less than the shortest code.
!  */
! int inflate_table(type, lens, codes, table, bits, work)
! codetype type;
! unsigned short FAR *lens;
! unsigned codes;
! code FAR * FAR *table;
! unsigned FAR *bits;
! unsigned short FAR *work;
! {
!     unsigned len;               /* a code's length in bits */
!     unsigned sym;               /* index of code symbols */
!     unsigned min, max;          /* minimum and maximum code lengths */
!     unsigned root;              /* number of index bits for root table */
!     unsigned curr;              /* number of index bits for current table */
!     unsigned drop;              /* code bits to drop for sub-table */
!     int left;                   /* number of prefix codes available */
!     unsigned used;              /* code entries in table used */
!     unsigned huff;              /* Huffman code */
!     unsigned incr;              /* for incrementing code, index */
!     unsigned fill;              /* index for replicating entries */
!     unsigned low;               /* low bits for current root entry */
!     unsigned mask;              /* mask for low root bits */
!     code here;                  /* table entry for duplication */
!     code FAR *next;             /* next available space in table */
!     const unsigned short FAR *base;     /* base value table to use */
!     const unsigned short FAR *extra;    /* extra bits table to use */
!     int end;                    /* use base and extra for symbol > end */
!     unsigned short count[MAXBITS+1];    /* number of codes of each length */
!     unsigned short offs[MAXBITS+1];     /* offsets in table for each length */
!     static const unsigned short lbase[31] = { /* Length codes 257..285 base */
          3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 15, 17, 19, 23, 27, 31,
          35, 43, 51, 59, 67, 83, 99, 115, 131, 163, 195, 227, 258, 0, 0};
!     static const unsigned short lext[31] = { /* Length codes 257..285 extra */
!         16, 16, 16, 16, 16, 16, 16, 16, 17, 17, 17, 17, 18, 18, 18, 18,
!         19, 19, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 16, 201, 203};
!     static const unsigned short dbase[32] = { /* Distance codes 0..29 base */
          1, 2, 3, 4, 5, 7, 9, 13, 17, 25, 33, 49, 65, 97, 129, 193,
          257, 385, 513, 769, 1025, 1537, 2049, 3073, 4097, 6145,
!         8193, 12289, 16385, 24577, 0, 0};
!     static const unsigned short dext[32] = { /* Distance codes 0..29 extra */
!         16, 16, 16, 16, 17, 17, 18, 18, 19, 19, 20, 20, 21, 21, 22, 22,
!         23, 23, 24, 24, 25, 25, 26, 26, 27, 27,
!         28, 28, 29, 29, 64, 64};
  
      /*
!        Process a set of code lengths to create a canonical Huffman code.  The
!        code lengths are lens[0..codes-1].  Each length corresponds to the
!        symbols 0..codes-1.  The Huffman code is generated by first sorting the
!        symbols by length from short to long, and retaining the symbol order
!        for codes with equal lengths.  Then the code starts with all zero bits
!        for the first code of the shortest length, and the codes are integer
!        increments for the same length, and zeros are appended as the length
!        increases.  For the deflate format, these bits are stored backwards
!        from their more natural integer increment ordering, and so when the
!        decoding tables are built in the large loop below, the integer codes
!        are incremented backwards.
! 
!        This routine assumes, but does not check, that all of the entries in
!        lens[] are in the range 0..MAXBITS.  The caller must assure this.
!        1..MAXBITS is interpreted as that code length.  zero means that that
!        symbol does not occur in this code.
! 
!        The codes are sorted by computing a count of codes for each length,
!        creating from that a table of starting indices for each length in the
!        sorted table, and then entering the symbols in order in the sorted
!        table.  The sorted table is work[], with that space being provided by
!        the caller.
! 
!        The length counts are used for other purposes as well, i.e. finding
!        the minimum and maximum length codes, determining if there are any
!        codes at all, checking for a valid set of lengths, and looking ahead
!        at length counts to determine sub-table sizes when building the
!        decoding tables.
       */
  
+     /* accumulate lengths for codes (assumes lens[] all in 0..MAXBITS) */
+     for (len = 0; len <= MAXBITS; len++)
+         count[len] = 0;
+     for (sym = 0; sym < codes; sym++)
+         count[lens[sym]]++;
+ 
+     /* bound code lengths, force root to be within code lengths */
+     root = *bits;
+     for (max = MAXBITS; max >= 1; max--)
+         if (count[max] != 0) break;
+     if (root > max) root = max;
+     if (max == 0) {                     /* no symbols to code at all */
+         here.op = (unsigned char)64;    /* invalid code marker */
+         here.bits = (unsigned char)1;
+         here.val = (unsigned short)0;
+         *(*table)++ = here;             /* make a table to force an error */
+         *(*table)++ = here;
+         *bits = 1;
+         return 0;     /* no symbols, but wait for decoding to report error */
+     }
+     for (min = 1; min <= MAXBITS; min++)
+         if (count[min] != 0) break;
+     if (root < min) root = min;
+ 
+     /* check for an over-subscribed or incomplete set of lengths */
+     left = 1;
+     for (len = 1; len <= MAXBITS; len++) {
+         left <<= 1;
+         left -= count[len];
+         if (left < 0) return -1;        /* over-subscribed */
+     }
+     if (left > 0 && (type == CODES || max != 1))
+         return -1;                      /* incomplete set */
+ 
+     /* generate offsets into symbol table for each length for sorting */
+     offs[1] = 0;
+     for (len = 1; len < MAXBITS; len++)
+         offs[len + 1] = offs[len] + count[len];
+ 
+     /* sort symbols by length, by symbol order within each length */
+     for (sym = 0; sym < codes; sym++)
+         if (lens[sym] != 0) work[offs[lens[sym]]++] = (unsigned short)sym;
  
!     /*
!        Create and fill in decoding tables.  In this loop, the table being
!        filled is at next and has curr index bits.  The code being used is huff
!        with length len.  That code is converted to an index by dropping drop
!        bits off of the bottom.  For codes where len is less than drop + curr,
!        those top drop + curr - len bits are incremented through all values to
!        fill the table with replicated entries.
! 
!        root is the number of index bits for the root table.  When len exceeds
!        root, sub-tables are created pointed to by the root entry with an index
!        of the low root bits of huff.  This is saved in low to check for when a
!        new sub-table should be started.  drop is zero when the root table is
!        being filled, and drop is root when sub-tables are being filled.
! 
!        When a new sub-table is needed, it is necessary to look ahead in the
!        code lengths to determine what size sub-table is needed.  The length
!        counts are used for this, and so count[] is decremented as codes are
!        entered in the tables.
! 
!        used keeps track of how many table entries have been allocated from the
!        provided *table space.  It is checked when a LENS table is being made
!        against the space in *table, ENOUGH, minus the maximum space needed by
!        the worst case distance code, MAXD.  This should never happen, but the
!        sufficiency of ENOUGH has not been proven exhaustively, hence the check.
!        This assumes that when type == LENS, bits == 9.
! 
!        sym increments through all symbols, and the loop terminates when
!        all codes of length max, i.e. all codes, have been processed.  This
!        routine permits incomplete codes, so another loop after this one fills
!        in the rest of the decoding tables with invalid code markers.
!      */
  
!     /* set up for code type */
!     switch (type) {
!     case CODES:
!         base = extra = work;    /* dummy value--not used */
!         end = 19;
          break;
!     case LENS:
!         base = lbase;
!         base -= 257;
!         extra = lext;
!         extra -= 257;
!         end = 256;
          break;
!     default:            /* DISTS */
!         base = dbase;
!         extra = dext;
!         end = -1;
!     }
! 
!     /* initialize state for loop */
!     huff = 0;                   /* starting code */
!     sym = 0;                    /* starting code symbol */
!     len = min;                  /* starting code length */
!     next = *table;              /* current table to fill in */
!     curr = root;                /* current table index bits */
!     drop = 0;                   /* current bits to drop from code for index */
!     low = (unsigned)(-1);       /* trigger new sub-table when len > root */
!     used = 1U << root;          /* use root table entries */
!     mask = used - 1;            /* mask for comparing low */
! 
!     /* check available table space */
!     if (type == LENS && used >= ENOUGH - MAXD)
!         return 1;
! 
!     /* process all codes and make table entries */
!     for (;;) {
!         /* create table entry */
!         here.bits = (unsigned char)(len - drop);
!         if ((int)(work[sym]) < end) {
!             here.op = (unsigned char)0;
!             here.val = work[sym];
!         }
!         else if ((int)(work[sym]) > end) {
!             here.op = (unsigned char)(extra[work[sym]]);
!             here.val = base[work[sym]];
!         }
!         else {
!             here.op = (unsigned char)(32 + 64);         /* end of block */
!             here.val = 0;
!         }
! 
!         /* replicate for those indices with low len bits equal to huff */
!         incr = 1U << (len - drop);
!         fill = 1U << curr;
!         min = fill;                 /* save offset to next table */
          do {
!             fill -= incr;
!             next[(huff >> drop) + fill] = here;
!         } while (fill != 0);
! 
!         /* backwards increment the len-bit code huff */
!         incr = 1U << (len - 1);
!         while (huff & incr)
!             incr >>= 1;
!         if (incr != 0) {
!             huff &= incr - 1;
!             huff += incr;
          }
          else
!             huff = 0;
  
!         /* go to next symbol, update count, len */
!         sym++;
!         if (--(count[len]) == 0) {
!             if (len == max) break;
!             len = lens[work[sym]];
!         }
! 
!         /* create new sub-table if needed */
!         if (len > root && (huff & mask) != low) {
!             /* if first time, transition to sub-tables */
!             if (drop == 0)
!                 drop = root;
! 
!             /* increment past last table */
!             next += min;            /* here min is 1 << curr */
! 
!             /* determine length of next table */
!             curr = len - drop;
!             left = (int)(1 << curr);
!             while (curr + drop < max) {
!                 left -= count[curr + drop];
!                 if (left <= 0) break;
!                 curr++;
!                 left <<= 1;
!             }
! 
!             /* check for enough space */
!             used += 1U << curr;
!             if (type == LENS && used >= ENOUGH - MAXD)
!                 return 1;
! 
!             /* point entry in root table to sub-table */
!             low = huff & mask;
!             (*table)[low].op = (unsigned char)curr;
!             (*table)[low].bits = (unsigned char)root;
!             (*table)[low].val = (unsigned short)(next - *table);
          }
      }
  
!     /*
!        Fill in rest of table for incomplete codes.  This loop is similar to the
!        loop above in incrementing huff for table indices.  It is assumed that
!        len is equal to curr + drop, so there is no loop needed to increment
!        through high index bits.  When the current sub-table is filled, the loop
!        drops back to the root table to fill in any remaining entries there.
!      */
!     here.op = (unsigned char)64;                /* invalid code marker */
!     here.bits = (unsigned char)(len - drop);
!     here.val = (unsigned short)0;
!     while (huff != 0) {
!         /* when done with sub-table, drop back to root table */
!         if (drop != 0 && (huff & mask) != low) {
!             drop = 0;
!             len = root;
!             next = *table;
!             here.bits = (unsigned char)len;
!         }
! 
!         /* put invalid code marker in table */
!         next[huff >> drop] = here;
! 
!         /* backwards increment the len-bit code huff */
!         incr = 1U << (len - 1);
!         while (huff & incr)
!             incr >>= 1;
!         if (incr != 0) {
!             huff &= incr - 1;
!             huff += incr;
          }
!         else
!             huff = 0;
      }
  
!     /* set return parameters */
!     *table += used;
!     *bits = root;
!     return 0;
  }
